/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software and related documentation ("Software") are intellectual
* property owned by TARA Systems and are copyright of TARA Systems.
* Any modification, copying, reproduction or redistribution of the Software in
* whole or in part by any means not in accordance with the End-User License
* Agreement for Embedded Wizard is expressly prohibited. The removal of this
* preamble is expressly prohibited.
* 
********************************************************************************
*
* DESCRIPTION:
*   This is an internal header of the Embedded Wizard Graphics Engine EWGFX. It
*   resolves the dependencies to the external graphics subsystem functionality
*   and replaces the missing functionality by Embedded Wizard own software pixel
*   driver.
*
*   The product of this header file is a set of arrays containing pointers to
*   external drawing functions or to the functions of the software pixel driver.
*
*   This header file also performs a rudimentary integrity test of the external
*   drawing and surface adaptation. If there are incomplete functionality, the
*   'C' preprocessor will report error messages.
*
*******************************************************************************/


/* Support of a platform specific screen surface? */
#if defined EwGfxLockScreenSurface ||                                          \
    defined EwGfxUnlockScreenSurface
  #define EwProvideScreenSurface
#endif


/* Support of a platform specific native surface? */
#if defined EwGfxCreateNativeSurface ||                                        \
    defined EwGfxCreateNativeSurfaceWrapped ||                                 \
    defined EwGfxCreateConstNativeSurface ||                                   \
    defined EwGfxDestroyNativeSurface ||                                       \
    defined EwGfxLockNativeSurface ||                                          \
    defined EwGfxUnlockNativeSurface
  #define EwProvideNativeSurface
#endif


/* Support of a platform specific index8 surface? */
#if defined EwGfxCreateIndex8Surface ||                                        \
    defined EwGfxCreateIndex8SurfaceWrapped ||                                 \
    defined EwGfxCreateConstIndex8Surface ||                                   \
    defined EwGfxDestroyIndex8Surface ||                                       \
    defined EwGfxLockIndex8Surface ||                                          \
    defined EwGfxUnlockIndex8Surface
  #define EwProvideIndex8Surface
#endif


/* Support of a platform specific alpha8 surface? */
#if defined EwGfxCreateAlpha8Surface ||                                        \
    defined EwGfxCreateAlpha8SurfaceWrapped ||                                 \
    defined EwGfxCreateConstAlpha8Surface ||                                   \
    defined EwGfxDestroyAlpha8Surface ||                                       \
    defined EwGfxLockAlpha8Surface ||                                          \
    defined EwGfxUnlockAlpha8Surface
  #define EwProvideAlpha8Surface
#endif


/* Support of a platform specific rgb565 surface? */
#if defined EwGfxCreateRGB565Surface ||                                        \
    defined EwGfxCreateRGB565SurfaceWrapped ||                                 \
    defined EwGfxCreateConstRGB565Surface ||                                   \
    defined EwGfxDestroyRGB565Surface ||                                       \
    defined EwGfxLockRGB565Surface ||                                          \
    defined EwGfxUnlockRGB565Surface
  #define EwProvideRGB565Surface
#endif


/* Verify the integrity of the screen surface adaptation */
#if defined EwProvideScreenSurface &&                                          \
    (                                                                          \
      !defined EwGfxLockScreenSurface ||                                       \
      !defined EwGfxUnlockScreenSurface                                        \
    )
  #error "Incomplete screen surface support. Some functions are missing."
#endif


/* Verify the integrity of the native surface adaptation */
#if defined EwProvideNativeSurface &&                                          \
    (                                                                          \
      !defined EwGfxCreateNativeSurface ||                                     \
      !defined EwGfxDestroyNativeSurface ||                                    \
      !defined EwGfxLockNativeSurface ||                                       \
      !defined EwGfxUnlockNativeSurface                                        \
    )
  #error "Incomplete native surface support. Some functions are missing."
#endif


/* Verify the integrity of the index8 surface adaptation */
#if defined EwProvideIndex8Surface &&                                          \
    (                                                                          \
      !defined EwGfxCreateIndex8Surface ||                                     \
      !defined EwGfxDestroyIndex8Surface ||                                    \
      !defined EwGfxLockIndex8Surface ||                                       \
      !defined EwGfxUnlockIndex8Surface                                        \
    )
  #error "Incomplete index8 surface support. Some functions are missing."
#endif


/* Verify the integrity of the alpha8 surface adaptation */
#if defined EwProvideAlpha8Surface &&                                          \
    (                                                                          \
      !defined EwGfxCreateAlpha8Surface ||                                     \
      !defined EwGfxDestroyAlpha8Surface ||                                    \
      !defined EwGfxLockAlpha8Surface ||                                       \
      !defined EwGfxUnlockAlpha8Surface                                        \
    )
  #error "Incomplete alpha8 surface support. Some functions are missing."
#endif


/* Verify the integrity of the rgb565 surface adaptation */
#if defined EwProvideRGB565Surface &&                                          \
    (                                                                          \
      !defined EwGfxCreateRGB565Surface ||                                     \
      !defined EwGfxDestroyRGB565Surface ||                                    \
      !defined EwGfxLockRGB565Surface ||                                       \
      !defined EwGfxUnlockRGB565Surface                                        \
    )
  #error "Incomplete rgb565 surface support. Some functions are missing."
#endif


/* Drawing operations use Native or optionally Screen surfaces as destination.
   Verify whether at least one of the surface types is available. */
#if defined EW_DONT_USE_NATIVE_SURFACES_AS_DESTINATION &&                      \
   !defined EW_USE_PIXEL_FORMAT_SCREEN
  #error "No surface type able to serve as destination in drawing operations."
#endif


/* Platform specific drawing operations involving screen surface as the
   destination? */
#if defined EwGfxScreenDrawLineSolid ||                                        \
    defined EwGfxScreenDrawLineSolidBlend ||                                   \
    defined EwGfxScreenFillSolid ||                                            \
    defined EwGfxScreenFillSolidBlend ||                                       \
    defined EwGfxScreenDrawLineGradient ||                                     \
    defined EwGfxScreenDrawLineGradientBlend ||                                \
    defined EwGfxScreenFillGradient ||                                         \
    defined EwGfxScreenFillGradientBlend ||                                    \
    defined EwGfxScreenFillLinearGradient ||                                   \
    defined EwGfxScreenFillLinearGradientBlend ||                              \
    defined EwGfxScreenPolygonSolid ||                                         \
    defined EwGfxScreenPolygonSolidBlend ||                                    \
    defined EwGfxScreenPolygonGradient ||                                      \
    defined EwGfxScreenPolygonGradientBlend ||                                 \
    defined EwGfxScreenPolygonLinearGradient ||                                \
    defined EwGfxScreenPolygonLinearGradientBlend ||                           \
    defined EwGfxScreenPolygonAntialiasedSolid ||                              \
    defined EwGfxScreenPolygonAntialiasedSolidBlend ||                         \
    defined EwGfxScreenPolygonAntialiasedGradient ||                           \
    defined EwGfxScreenPolygonAntialiasedGradientBlend ||                      \
    defined EwGfxScreenPolygonAntialiasedLinearGradient ||                     \
    defined EwGfxScreenPolygonAntialiasedLinearGradientBlend
  #define EwNeedScreenSurface
#endif


/* Platform specific drawing operations involving screen surface as the 
   destination and native surface as source? */
#if defined EwGfxScreenCopyNative ||                                           \
    defined EwGfxScreenCopyNativeBlend ||                                      \
    defined EwGfxScreenCopyNativeGradient ||                                   \
    defined EwGfxScreenCopyNativeGradientBlend ||                              \
    defined EwGfxScreenCopyNativeLinearGradient ||                             \
    defined EwGfxScreenCopyNativeLinearGradientBlend ||                        \
    defined EwGfxScreenTileNative ||                                           \
    defined EwGfxScreenTileNativeBlend ||                                      \
    defined EwGfxScreenTileNativeSolid ||                                      \
    defined EwGfxScreenTileNativeSolidBlend ||                                 \
    defined EwGfxScreenTileNativeGradient ||                                   \
    defined EwGfxScreenTileNativeGradientBlend ||                              \
    defined EwGfxScreenTileNativeLinearGradient ||                             \
    defined EwGfxScreenTileNativeLinearGradientBlend ||                        \
    defined EwGfxScreenWarpPerspNative ||                                      \
    defined EwGfxScreenWarpPerspNativeBlend ||                                 \
    defined EwGfxScreenWarpPerspNativeGradient ||                              \
    defined EwGfxScreenWarpPerspNativeGradientBlend ||                         \
    defined EwGfxScreenWarpPerspNativeFilter ||                                \
    defined EwGfxScreenWarpPerspNativeFilterBlend ||                           \
    defined EwGfxScreenWarpPerspNativeFilterGradient ||                        \
    defined EwGfxScreenWarpPerspNativeFilterGradientBlend ||                   \
    defined EwGfxScreenWarpAffineNative ||                                     \
    defined EwGfxScreenWarpAffineNativeBlend ||                                \
    defined EwGfxScreenWarpAffineNativeGradient ||                             \
    defined EwGfxScreenWarpAffineNativeGradientBlend ||                        \
    defined EwGfxScreenWarpAffineNativeFilter ||                               \
    defined EwGfxScreenWarpAffineNativeFilterBlend ||                          \
    defined EwGfxScreenWarpAffineNativeFilterGradient ||                       \
    defined EwGfxScreenWarpAffineNativeFilterGradientBlend ||                  \
    defined EwGfxScreenScaleNative ||                                          \
    defined EwGfxScreenScaleNativeBlend ||                                     \
    defined EwGfxScreenScaleNativeGradient ||                                  \
    defined EwGfxScreenScaleNativeGradientBlend ||                             \
    defined EwGfxScreenScaleNativeLinearGradient ||                            \
    defined EwGfxScreenScaleNativeLinearGradientBlend ||                       \
    defined EwGfxScreenScaleNativeFilter ||                                    \
    defined EwGfxScreenScaleNativeFilterBlend ||                               \
    defined EwGfxScreenScaleNativeFilterGradient ||                            \
    defined EwGfxScreenScaleNativeFilterGradientBlend ||                       \
    defined EwGfxScreenScaleNativeFilterLinearGradient ||                      \
    defined EwGfxScreenScaleNativeFilterLinearGradientBlend ||                 \
    defined EwGfxScreenFilterNativeBlur ||                                     \
    defined EwGfxScreenFilterNativeMask ||                                     \
    defined EwGfxScreenFilterNativeBlurMask ||                                 \
    defined EwGfxScreenFilterNativeTintByOpacity ||                            \
    defined EwGfxScreenFilterNativeTintByLuminance ||                          \
    defined EwGfxScreenFilterNativeBlurTintByOpacity ||                        \
    defined EwGfxScreenFilterNativeBlurTintByLuminance ||                      \
    defined EwGfxScreenFilterNativeMaskTintByOpacity ||                        \
    defined EwGfxScreenFilterNativeMaskTintByLuminance ||                      \
    defined EwGfxScreenFilterNativeBlurMaskTintByOpacity ||                    \
    defined EwGfxScreenFilterNativeBlurMaskTintByLuminance
  #define EwNeedScreenSurface
  #define EwNeedNativeSurface
#endif


/* Platform specific drawing operations involving native surface as source and
   destination? */
#if defined EwGfxDrawLineSolid ||                                              \
    defined EwGfxDrawLineSolidBlend ||                                         \
    defined EwGfxFillSolid ||                                                  \
    defined EwGfxFillSolidBlend ||                                             \
    defined EwGfxDrawLineGradient ||                                           \
    defined EwGfxDrawLineGradientBlend ||                                      \
    defined EwGfxFillGradient ||                                               \
    defined EwGfxFillGradientBlend ||                                          \
    defined EwGfxFillLinearGradient ||                                         \
    defined EwGfxFillLinearGradientBlend ||                                    \
    defined EwGfxCopyNative ||                                                 \
    defined EwGfxCopyNativeBlend ||                                            \
    defined EwGfxCopyNativeGradient ||                                         \
    defined EwGfxCopyNativeGradientBlend ||                                    \
    defined EwGfxCopyNativeLinearGradient ||                                   \
    defined EwGfxCopyNativeLinearGradientBlend ||                              \
    defined EwGfxTileNative ||                                                 \
    defined EwGfxTileNativeBlend ||                                            \
    defined EwGfxTileNativeSolid ||                                            \
    defined EwGfxTileNativeSolidBlend ||                                       \
    defined EwGfxTileNativeGradient ||                                         \
    defined EwGfxTileNativeGradientBlend ||                                    \
    defined EwGfxTileNativeLinearGradient ||                                   \
    defined EwGfxTileNativeLinearGradientBlend ||                              \
    defined EwGfxWarpPerspNative ||                                            \
    defined EwGfxWarpPerspNativeBlend ||                                       \
    defined EwGfxWarpPerspNativeGradient ||                                    \
    defined EwGfxWarpPerspNativeGradientBlend ||                               \
    defined EwGfxWarpPerspNativeFilter ||                                      \
    defined EwGfxWarpPerspNativeFilterBlend ||                                 \
    defined EwGfxWarpPerspNativeFilterGradient ||                              \
    defined EwGfxWarpPerspNativeFilterGradientBlend ||                         \
    defined EwGfxWarpAffineNative ||                                           \
    defined EwGfxWarpAffineNativeBlend ||                                      \
    defined EwGfxWarpAffineNativeGradient ||                                   \
    defined EwGfxWarpAffineNativeGradientBlend ||                              \
    defined EwGfxWarpAffineNativeFilter ||                                     \
    defined EwGfxWarpAffineNativeFilterBlend ||                                \
    defined EwGfxWarpAffineNativeFilterGradient ||                             \
    defined EwGfxWarpAffineNativeFilterGradientBlend ||                        \
    defined EwGfxScaleNative ||                                                \
    defined EwGfxScaleNativeBlend ||                                           \
    defined EwGfxScaleNativeGradient ||                                        \
    defined EwGfxScaleNativeGradientBlend ||                                   \
    defined EwGfxScaleNativeLinearGradient ||                                  \
    defined EwGfxScaleNativeLinearGradientBlend ||                             \
    defined EwGfxScaleNativeFilter ||                                          \
    defined EwGfxScaleNativeFilterBlend ||                                     \
    defined EwGfxScaleNativeFilterGradient ||                                  \
    defined EwGfxScaleNativeFilterGradientBlend ||                             \
    defined EwGfxScaleNativeFilterLinearGradient ||                            \
    defined EwGfxScaleNativeFilterLinearGradientBlend ||                       \
    defined EwGfxPolygonSolid ||                                               \
    defined EwGfxPolygonSolidBlend ||                                          \
    defined EwGfxPolygonGradient ||                                            \
    defined EwGfxPolygonGradientBlend ||                                       \
    defined EwGfxPolygonLinearGradient ||                                      \
    defined EwGfxPolygonLinearGradientBlend ||                                 \
    defined EwGfxPolygonAntialiasedSolid ||                                    \
    defined EwGfxPolygonAntialiasedSolidBlend ||                               \
    defined EwGfxPolygonAntialiasedGradient ||                                 \
    defined EwGfxPolygonAntialiasedGradientBlend ||                            \
    defined EwGfxPolygonAntialiasedLinearGradient ||                           \
    defined EwGfxPolygonAntialiasedLinearGradientBlend ||                      \
    defined EwGfxFilterNativeBlur ||                                           \
    defined EwGfxFilterNativeMask ||                                           \
    defined EwGfxFilterNativeBlurMask ||                                       \
    defined EwGfxFilterNativeTintByOpacity ||                                  \
    defined EwGfxFilterNativeTintByLuminance ||                                \
    defined EwGfxFilterNativeBlurTintByOpacity ||                              \
    defined EwGfxFilterNativeBlurTintByLuminance ||                            \
    defined EwGfxFilterNativeMaskTintByOpacity ||                              \
    defined EwGfxFilterNativeMaskTintByLuminance ||                            \
    defined EwGfxFilterNativeBlurMaskTintByOpacity ||                          \
    defined EwGfxFilterNativeBlurMaskTintByLuminance
  #define EwNeedNativeSurface
#endif


/* Platform specific drawing operations involving index8 surface as source and
   the screen surface as destination? */
#if defined EwGfxScreenCopyIndex8 ||                                           \
    defined EwGfxScreenCopyIndex8Blend ||                                      \
    defined EwGfxScreenCopyIndex8Gradient ||                                   \
    defined EwGfxScreenCopyIndex8GradientBlend ||                              \
    defined EwGfxScreenCopyIndex8LinearGradient ||                             \
    defined EwGfxScreenCopyIndex8LinearGradientBlend ||                        \
    defined EwGfxScreenTileIndex8 ||                                           \
    defined EwGfxScreenTileIndex8Blend ||                                      \
    defined EwGfxScreenTileIndex8Solid ||                                      \
    defined EwGfxScreenTileIndex8SolidBlend ||                                 \
    defined EwGfxScreenTileIndex8Gradient ||                                   \
    defined EwGfxScreenTileIndex8GradientBlend ||                              \
    defined EwGfxScreenTileIndex8LinearGradient ||                             \
    defined EwGfxScreenTileIndex8LinearGradientBlend ||                        \
    defined EwGfxScreenWarpPerspIndex8 ||                                      \
    defined EwGfxScreenWarpPerspIndex8Blend ||                                 \
    defined EwGfxScreenWarpPerspIndex8Gradient ||                              \
    defined EwGfxScreenWarpPerspIndex8GradientBlend ||                         \
    defined EwGfxScreenWarpPerspIndex8Filter ||                                \
    defined EwGfxScreenWarpPerspIndex8FilterBlend ||                           \
    defined EwGfxScreenWarpPerspIndex8FilterGradient ||                        \
    defined EwGfxScreenWarpPerspIndex8FilterGradientBlend ||                   \
    defined EwGfxScreenWarpAffineIndex8 ||                                     \
    defined EwGfxScreenWarpAffineIndex8Blend ||                                \
    defined EwGfxScreenWarpAffineIndex8Gradient ||                             \
    defined EwGfxScreenWarpAffineIndex8GradientBlend ||                        \
    defined EwGfxScreenWarpAffineIndex8Filter ||                               \
    defined EwGfxScreenWarpAffineIndex8FilterBlend ||                          \
    defined EwGfxScreenWarpAffineIndex8FilterGradient ||                       \
    defined EwGfxScreenWarpAffineIndex8FilterGradientBlend ||                  \
    defined EwGfxScreenScaleIndex8 ||                                          \
    defined EwGfxScreenScaleIndex8Blend ||                                     \
    defined EwGfxScreenScaleIndex8Gradient ||                                  \
    defined EwGfxScreenScaleIndex8GradientBlend ||                             \
    defined EwGfxScreenScaleIndex8LinearGradient ||                            \
    defined EwGfxScreenScaleIndex8LinearGradientBlend ||                       \
    defined EwGfxScreenScaleIndex8Filter ||                                    \
    defined EwGfxScreenScaleIndex8FilterBlend ||                               \
    defined EwGfxScreenScaleIndex8FilterGradient ||                            \
    defined EwGfxScreenScaleIndex8FilterGradientBlend ||                       \
    defined EwGfxScreenScaleIndex8FilterLinearGradient ||                      \
    defined EwGfxScreenScaleIndex8FilterLinearGradientBlend ||                 \
    defined EwGfxScreenFilterIndex8Blur ||                                     \
    defined EwGfxScreenFilterIndex8Mask ||                                     \
    defined EwGfxScreenFilterIndex8BlurMask ||                                 \
    defined EwGfxScreenFilterIndex8TintByOpacity ||                            \
    defined EwGfxScreenFilterIndex8TintByLuminance ||                          \
    defined EwGfxScreenFilterIndex8BlurTintByOpacity ||                        \
    defined EwGfxScreenFilterIndex8BlurTintByLuminance ||                      \
    defined EwGfxScreenFilterIndex8MaskTintByOpacity ||                        \
    defined EwGfxScreenFilterIndex8MaskTintByLuminance ||                      \
    defined EwGfxScreenFilterIndex8BlurMaskTintByOpacity ||                    \
    defined EwGfxScreenFilterIndex8BlurMaskTintByLuminance
  #define EwNeedScreenSurface
  #define EwNeedIndex8Surface
#endif


/* Platform specific drawing operations involving index8 surface as source and
   the native surface as destination? */
#if defined EwGfxCopyIndex8 ||                                                 \
    defined EwGfxCopyIndex8Blend ||                                            \
    defined EwGfxCopyIndex8Gradient ||                                         \
    defined EwGfxCopyIndex8GradientBlend ||                                    \
    defined EwGfxCopyIndex8LinearGradient ||                                   \
    defined EwGfxCopyIndex8LinearGradientBlend ||                              \
    defined EwGfxTileIndex8 ||                                                 \
    defined EwGfxTileIndex8Blend ||                                            \
    defined EwGfxTileIndex8Solid ||                                            \
    defined EwGfxTileIndex8SolidBlend ||                                       \
    defined EwGfxTileIndex8Gradient ||                                         \
    defined EwGfxTileIndex8GradientBlend ||                                    \
    defined EwGfxTileIndex8LinearGradient ||                                   \
    defined EwGfxTileIndex8LinearGradientBlend ||                              \
    defined EwGfxWarpPerspIndex8 ||                                            \
    defined EwGfxWarpPerspIndex8Blend ||                                       \
    defined EwGfxWarpPerspIndex8Gradient ||                                    \
    defined EwGfxWarpPerspIndex8GradientBlend ||                               \
    defined EwGfxWarpPerspIndex8Filter ||                                      \
    defined EwGfxWarpPerspIndex8FilterBlend ||                                 \
    defined EwGfxWarpPerspIndex8FilterGradient ||                              \
    defined EwGfxWarpPerspIndex8FilterGradientBlend ||                         \
    defined EwGfxWarpAffineIndex8 ||                                           \
    defined EwGfxWarpAffineIndex8Blend ||                                      \
    defined EwGfxWarpAffineIndex8Gradient ||                                   \
    defined EwGfxWarpAffineIndex8GradientBlend ||                              \
    defined EwGfxWarpAffineIndex8Filter ||                                     \
    defined EwGfxWarpAffineIndex8FilterBlend ||                                \
    defined EwGfxWarpAffineIndex8FilterGradient ||                             \
    defined EwGfxWarpAffineIndex8FilterGradientBlend ||                        \
    defined EwGfxScaleIndex8 ||                                                \
    defined EwGfxScaleIndex8Blend ||                                           \
    defined EwGfxScaleIndex8Gradient ||                                        \
    defined EwGfxScaleIndex8GradientBlend ||                                   \
    defined EwGfxScaleIndex8LinearGradient ||                                  \
    defined EwGfxScaleIndex8LinearGradientBlend ||                             \
    defined EwGfxScaleIndex8Filter ||                                          \
    defined EwGfxScaleIndex8FilterBlend ||                                     \
    defined EwGfxScaleIndex8FilterGradient ||                                  \
    defined EwGfxScaleIndex8FilterGradientBlend ||                             \
    defined EwGfxScaleIndex8FilterLinearGradient ||                            \
    defined EwGfxScaleIndex8FilterLinearGradientBlend ||                       \
    defined EwGfxFilterIndex8Blur ||                                           \
    defined EwGfxFilterIndex8Mask ||                                           \
    defined EwGfxFilterIndex8BlurMask ||                                       \
    defined EwGfxFilterIndex8TintByOpacity ||                                  \
    defined EwGfxFilterIndex8TintByLuminance ||                                \
    defined EwGfxFilterIndex8BlurTintByOpacity ||                              \
    defined EwGfxFilterIndex8BlurTintByLuminance ||                            \
    defined EwGfxFilterIndex8MaskTintByOpacity ||                              \
    defined EwGfxFilterIndex8MaskTintByLuminance ||                            \
    defined EwGfxFilterIndex8BlurMaskTintByOpacity ||                          \
    defined EwGfxFilterIndex8BlurMaskTintByLuminance
  #define EwNeedNativeSurface
  #define EwNeedIndex8Surface
#endif


/* Platform specific drawing operations involving alpha8 surface as source and
   the screen surface as destination? */
#if defined EwGfxScreenCopyAlpha8Gradient ||                                   \
    defined EwGfxScreenCopyAlpha8GradientBlend ||                              \
    defined EwGfxScreenCopyAlpha8LinearGradient ||                             \
    defined EwGfxScreenCopyAlpha8LinearGradientBlend ||                        \
    defined EwGfxScreenTileAlpha8Solid ||                                      \
    defined EwGfxScreenTileAlpha8SolidBlend ||                                 \
    defined EwGfxScreenTileAlpha8Gradient ||                                   \
    defined EwGfxScreenTileAlpha8GradientBlend ||                              \
    defined EwGfxScreenTileAlpha8LinearGradient ||                             \
    defined EwGfxScreenTileAlpha8LinearGradientBlend ||                        \
    defined EwGfxScreenWarpPerspAlpha8Gradient ||                              \
    defined EwGfxScreenWarpPerspAlpha8GradientBlend ||                         \
    defined EwGfxScreenWarpPerspAlpha8FilterGradient ||                        \
    defined EwGfxScreenWarpPerspAlpha8FilterGradientBlend ||                   \
    defined EwGfxScreenWarpAffineAlpha8Gradient ||                             \
    defined EwGfxScreenWarpAffineAlpha8GradientBlend ||                        \
    defined EwGfxScreenWarpAffineAlpha8FilterGradient ||                       \
    defined EwGfxScreenWarpAffineAlpha8FilterGradientBlend ||                  \
    defined EwGfxScreenScaleAlpha8Gradient ||                                  \
    defined EwGfxScreenScaleAlpha8GradientBlend ||                             \
    defined EwGfxScreenScaleAlpha8FilterGradient ||                            \
    defined EwGfxScreenScaleAlpha8FilterGradientBlend ||                       \
    defined EwGfxScreenScaleAlpha8LinearGradient ||                            \
    defined EwGfxScreenScaleAlpha8LinearGradientBlend ||                       \
    defined EwGfxScreenScaleAlpha8FilterLinearGradient ||                      \
    defined EwGfxScreenScaleAlpha8FilterLinearGradientBlend ||                 \
    defined EwGfxScreenFilterAlpha8Blur ||                                     \
    defined EwGfxScreenFilterAlpha8Mask ||                                     \
    defined EwGfxScreenFilterAlpha8BlurMask ||                                 \
    defined EwGfxScreenFilterAlpha8TintByOpacity ||                            \
    defined EwGfxScreenFilterAlpha8TintByLuminance ||                          \
    defined EwGfxScreenFilterAlpha8BlurTintByOpacity ||                        \
    defined EwGfxScreenFilterAlpha8BlurTintByLuminance ||                      \
    defined EwGfxScreenFilterAlpha8MaskTintByOpacity ||                        \
    defined EwGfxScreenFilterAlpha8MaskTintByLuminance ||                      \
    defined EwGfxScreenFilterAlpha8BlurMaskTintByOpacity ||                    \
    defined EwGfxScreenFilterAlpha8BlurMaskTintByLuminance
  #define EwNeedScreenSurface
  #define EwNeedAlpha8Surface
#endif


/* Platform specific drawing operations involving alpha8 surface as source and
   the native surface as destination? */
#if defined EwGfxCopyAlpha8Gradient ||                                         \
    defined EwGfxCopyAlpha8GradientBlend ||                                    \
    defined EwGfxCopyAlpha8LinearGradient ||            RGB565BlurTintByLuminance,
  EwGfxFilterRGB565MaskTintByLuminance,              EwGfxFilterRGB565BlurMaskTintByLuminance,

#ifdef EW_USE_PIXEL_FORMAT_SCREEN
  0,                                                 EwGfxScreenFilterNativeBlur,
  EwGfxScreenFilterNativeMask,                       EwGfxScreenFilterNativeBlurMask,
  EwGfxScreenFilterNativeTintByOpacity,              EwGfxScreenFilterNativeBlurTintByOpacity,
  EwGfxScreenFilterNativeMaskTintByOpacity,          EwGfxScreenFilterNativeBlurMaskTintByOpacity,
  EwGfxScreenFilterNativeTintByLuminance,            EwGfxScreenFilterNativeBlurTintByLuminance,
  EwGfxScreenFilterNativeMaskTintByLuminance,        EwGfxScreenFilterNativeBlurMaskTintByLuminance,

  0,                                                 EwGfxScreenFilterIndex8Blur,
  EwGfxScreenFilterIndex8Mask,                       EwGfxScreenFilterIndex8BlurMask,
  EwGfxScreenFilterIndex8TintByOpacity,              EwGfxScreenFilterIndex8BlurTintByOpacity,
  EwGfxScreenFilterIndex8MaskTintByOpacity,          EwGfxScreenFilterIndex8BlurMaskTintByOpacity,
  EwGfxScreenFilterIndex8TintByLuminance,            EwGfxScreenFilterIndex8BlurTintByLuminance,
  EwGfxScreenFilterIndex8MaskTintByLuminance,        EwGfxScreenFilterIndex8BlurMaskTintByLuminance,

  0,                                                 EwGfxScreenFilterAlpha8Blur,
  EwGfxScreenFilterAlpha8Mask,                       EwGfxScreenFilterAlpha8BlurMask,
  EwGfxScreenFilterAlpha8TintByOpacity,              EwGfxScreenFilterAlpha8BlurTintByOpacity,
  EwGfxScreenFilterAlpha8MaskTintByOpacity,          EwGfxScreenFilterAlpha8BlurMaskTintByOpacity,
  EwGfxScreenFilterAlpha8TintByLuminance,            EwGfxScreenFilterAlpha8BlurTintByLuminance,
  EwGfxScreenFilterAlpha8MaskTintByLuminance,        EwGfxScreenFilterAlpha8BlurMaskTintByLuminance,

  0,                                                 EwGfxScreenFilterRGB565Blur,
  EwGfxScreenFilterRGB565Mask,                       EwGfxScreenFilterRGB565BlurMask,
  EwGfxScreenFilterRGB565TintByOpacity,              EwGfxScreenFilterRGB565BlurTintByOpacity,
  EwGfxScreenFilterRGB565MaskTintByOpacity,          EwGfxScreenFilterRGB565BlurMaskTintByOpacity,
  EwGfxScreenFilterRGB565TintByLuminance,            EwGfxScreenFilterRGB565BlurTintByLuminance,
  EwGfxScreenFilterRGB565MaskTintByLuminance,        EwGfxScreenFilterRGB565BlurMaskTintByLuminance,
#endif

  0
};


/* Following workers are compatible to mask/tint workers but they don't perform 
   any mask/tint operations. The workers only copy the pixel between the source
   and the destination by additionally modulating the opacity and/or applying
   alpha-blending. These workers are used only in conjunction with the blur/mask
   filter, when an area outside an inverted mask is processed. In such case the
   affected area can be transfered directly into the destination surface without
   any mask filter. Please note: like all other mask/tint workers, the functions
   are able to write column-wise into the destination while reading row-wise the
   source. */
static const XMaskTintWorker NoFilterWorkers[] =
{
  EwFilterNative,

#ifdef EW_USE_PIXEL_FORMAT_SCREEN
  EwScreenFilterNative,
#endif

  0
};

/* For missing graphics subsystem functionality, register alternative software
   emulated functions */
static const XMaskTintWorker MaskTintWorkers[] =
{
  0,                                                 EwFilterNativeMask,
  EwFilterNativeTintByOpacity,                       EwFilterNativeMaskTintByOpacity,
  EwFilterNativeTintByLuminance,                     EwFilterNativeMaskTintByLuminance,

  0,                                                 EwFilterIndex8Mask,
  EwFilterIndex8TintByOpacity,                       EwFilterIndex8MaskTintByOpacity,
  EwFilterIndex8TintByLuminance,                     EwFilterIndex8MaskTintByLuminance,

  0,                                                 EwFilterAlpha8Mask,
  EwFilterAlpha8TintByOpacity,                       EwFilterAlpha8MaskTintByOpacity,
  EwFilterAlpha8TintByLuminance,                     EwFilterAlpha8MaskTintByLuminance,

  0,                                                 EwFilterRGB565Mask,
  EwFilterRGB565TintByOpacity,                       EwFilterRGB565MaskTintByOpacity,
  EwFilterRGB565TintByLuminance,                     EwFilterRGB565MaskTintByLuminance,

#ifdef EW_USE_PIXEL_FORMAT_SCREEN
  0,                                                 EwScreenFilterNativeMask,
  EwScreenFilterNativeTintByOpacity,                 EwScreenFilterNativeMaskTintByOpacity,
  EwScreenFilterNativeTintByLuminance,               EwScreenFilterNativeMaskTintByLuminance,

  0,                                                 EwScreenFilterIndex8Mask,
  EwScreenFilterIndex8TintByOpacity,                 EwScreenFilterIndex8MaskTintByOpacity,
  EwScreenFilterIndex8TintByLuminance,               EwScreenFilterIndex8MaskTintByLuminance,

  0,                                                 EwScreenFilterAlpha8Mask,
  EwScreenFilterAlpha8TintByOpacity,                 EwScreenFilterAlpha8MaskTintByOpacity,
  EwScreenFilterAlpha8TintByLuminance,               EwScreenFilterAlpha8MaskTintByLuminance,

  0,                                                 EwScreenFilterRGB565Mask,
  EwScreenFilterRGB565TintByOpacity,                 EwScreenFilterRGB565MaskTintByOpacity,
  EwScreenFilterRGB565TintByLuminance,               EwScreenFilterRGB565MaskTintByLuminance,
#endif

  0
};


/* For missing graphics subsystem functionality, register alternative software
   emulated functions */
static const XBlurWorker BlurWorkers[] =
{
  EwFilterNativeBlur,
  EwFilterIndex8Blur,
  0,
  EwFilterRGB565Blur,
  EwAlpha8FilterAlpha8Blur,

#ifdef EW_USE_PIXEL_FORMAT_SCREEN
  EwScreenFilterNativeBlur,
#endif

  0
};


/* pba */
